package com.slprojects.sl_videocompressor;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.media.MediaMetadataRetriever;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.arthenica.ffmpegkit.FFmpegKit;
import com.arthenica.ffmpegkit.FFmpegSession;
import com.arthenica.ffmpegkit.ReturnCode;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

public class MainActivity extends AppCompatActivity implements View.OnClickListener {

    private Button chooseFileBtn;
    private TextView videoInformations;
    private String video_url;
    private LinearLayout weHaveTheVideo;
    public Spinner sizeUnit;
    private EditText targetFileSizeInput;
    public Spinner encodeQuality;
    public Button encodeVideoBtn;
    public CheckBox deleteSoundTrack;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // On retire le titre de l'application
        getSupportActionBar().hide();

        // On va vérifier que l'on a bien les permissions
        if(ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED)
        {
            // On demande les permissions
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, 1);
        }
        if(ContextCompat.checkSelfPermission(this, Manifest.permission.READ_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED)
        {
            // On demande les permissions
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.READ_EXTERNAL_STORAGE}, 1);
        }

        chooseFileBtn = findViewById(R.id.chooseFileBtn);
        chooseFileBtn.setOnClickListener((View.OnClickListener) this);
        videoInformations = findViewById(R.id.videoInformations);
        weHaveTheVideo = findViewById(R.id.weHaveTheVideo);

        sizeUnit = findViewById(R.id.sizeUnit);
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.size_units, android.R.layout.simple_spinner_item);
        // Specify the layout to use when the list of choices appears
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        sizeUnit.setAdapter(adapter);

        encodeQuality = findViewById(R.id.encodeQuality);
        targetFileSizeInput = findViewById(R.id.targetFileSizeInput);
        ArrayAdapter<CharSequence> adapter2 = ArrayAdapter.createFromResource(this, R.array.encode_qualities, android.R.layout.simple_spinner_item);
        // Specify the layout to use when the list of choices appears
        adapter2.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        encodeQuality.setAdapter(adapter2);

        encodeVideoBtn = findViewById(R.id.encodeVideoBtn);
        encodeVideoBtn.setOnClickListener((View.OnClickListener) this);
        deleteSoundTrack = findViewById(R.id.deleteSoundTrack);
    }

    @Override
    public void onClick(View v){
        if(v == chooseFileBtn){
            // On va lancer l'activité de choix de fichier
            Intent intent = new Intent(
                    Intent.ACTION_PICK,
                    android.provider.MediaStore.Video.Media.EXTERNAL_CONTENT_URI);
            intent.setType("video/*");
            startActivityForResult(intent, 123);
        }else if(v == encodeVideoBtn){
            Log.i("MainActivity", "Encode video");
            // On va vérifier que l'utilisateur a renté une taille de fichier
            if(targetFileSizeInput.getText().toString().equals("")){
                Toast.makeText(this, "Veuillez rentrer une taille de fichier", Toast.LENGTH_SHORT).show();
                return;
            }

            // Mais aussi qu'il a bien choisir une qualité
            if(encodeQuality.getSelectedItem().toString().equals("")){
                Toast.makeText(this, "Veuillez choisir une qualité", Toast.LENGTH_SHORT).show();
                return;
            }

            // On peut lancer l'encodage
            encodeVideo encodage = new encodeVideo(this, video_url, Integer.parseInt(targetFileSizeInput.getText().toString()), encodeQuality.getSelectedItem().toString());
            encodage.execute();
        }
    }

    // Overriding the method onActivityResult()
    // to get the video Uri form intent.
    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (resultCode == RESULT_OK) {
            if (requestCode == 123) {
                if (data != null) {
                    // get the video Uri
                    Uri uri = data.getData();
                    try {
                        // get the file from the Uri using getFileFromUri() method present
                        // in FileUils.java
                        File video_file = FileUtils.getFileFromUri(this, uri);

                        // get the absolute path of the video file. We will require
                        // this as an input argument in
                        // the ffmpeg command.
                        video_url = video_file.getAbsolutePath();
                        aVideoHasBeenChoosen();

                    } catch (Exception e) {
                        Toast.makeText(this, "Error", Toast.LENGTH_SHORT).show();
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    public void aVideoHasBeenChoosen() {
        String video_name = video_url.substring(video_url.lastIndexOf("/") + 1);
        chooseFileBtn.setText(video_name);
        String affichage = "Nom : " + video_name + "\n" + "Durée : " + getVideoDuration(video_url) + "\n" + "Taille : " + getVideoSize(video_url);
        videoInformations.setText(affichage);
        videoInformations.setMaxLines(affichage.split("\n").length);

        weHaveTheVideo.setVisibility(View.VISIBLE);
    }

    // Fonction affichages
    public String getVideoDuration(String video_url) {
        MediaMetadataRetriever retriever = new MediaMetadataRetriever();
        retriever.setDataSource(video_url);
        String time = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_DURATION);
        long timeInMillisec = Long.parseLong(time);
        long duration = timeInMillisec / 1000;
        long hours = duration / 3600;
        long minutes = (duration - hours * 3600) / 60;
        long seconds = duration - (hours * 3600 + minutes * 60);
        return String.format("%02d:%02d:%02d", hours, minutes, seconds);
    }

    public String getVideoSize(String video_url) {
        File file = new File(video_url);
        long size = file.length();
        return String.format("%d Mo", size / 1000000);
    }

    // Fonction d'encodage
    public Integer getVideoDurationInSeconds(String video_url) {
        MediaMetadataRetriever retriever = new MediaMetadataRetriever();
        retriever.setDataSource(video_url);
        String time = retriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_DURATION);
        long timeInMillisec = Long.parseLong(time);
        return (int) (timeInMillisec / 1000);
    }

    public Integer getVideoSizeInBytes(String video_url) {
        File file = new File(video_url);
        long size = file.length();
        return (int) size;
    }
}