package com.slprojects.sl_videocompressor;

import android.os.AsyncTask;
import android.os.Environment;
import android.util.Log;
import android.widget.Toast;

import com.arthenica.ffmpegkit.FFmpegKit;
import com.arthenica.ffmpegkit.FFmpegSession;
import com.arthenica.ffmpegkit.ReturnCode;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

public class encodeVideo extends AsyncTask<String, Void, Void> {
    private final MainActivity activity;
    private final String video_url;
    private final Integer targetFileSize;
    private final String quality;

    public encodeVideo(MainActivity activity, String video_url, Integer targetFileSize, String quality) {
        this.activity = activity;
        this.video_url = video_url;
        this.targetFileSize = targetFileSize;
        this.quality = quality;
    }

    @Override
    protected Void doInBackground(String... params) {
        // On récupère la taille cible du fichier selon l'unité choisie
        Integer targetFileSizeInKiloBit = 0;
        if(activity.sizeUnit.getSelectedItem().toString().equals("Mo")) {
            targetFileSizeInKiloBit = targetFileSize * 1000 * 8;
        }else if(activity.sizeUnit.getSelectedItem().toString().equals("MB")) {
            targetFileSizeInKiloBit = targetFileSize * 1000 * 8;
        }else{
            activity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Toast.makeText(activity, "Unité de taille \"" + activity.sizeUnit.getSelectedItem().toString() + "\" non reconnue. M", Toast.LENGTH_SHORT).show();
                }
            });
        }
        //targetFileSizeInKiloBit = (int) (targetFileSizeInKiloBit * (9 / 10));
        Log.i("targetFileSizeInKiloBit", targetFileSizeInKiloBit.toString());

        // On récupère la durée de la vidéo
        Integer videoDurationInSeconds = activity.getVideoDurationInSeconds(video_url);
        Log.i("videoDurationInSeconds", videoDurationInSeconds.toString());

        // On regarde si on garde l'audio ou pas
        Integer audioBitrate = 0; // en kBits
        if(!activity.deleteSoundTrack.isChecked()) {
            audioBitrate = 128;
        }

        // On va calculer le bitrate de la video en fonction de la bande son
        Integer videoBitrate = (int) ((targetFileSizeInKiloBit / videoDurationInSeconds) - audioBitrate);
        Log.i("videoBitrate", videoBitrate.toString());

        // On regarde si a notre dossier de créé
        File folder = new File(Environment.getExternalStorageDirectory() + "/SL-Video Compressor/");
        if(!folder.exists()) {
            Log.w("folder", Environment.getExternalStorageDirectory() + "/SL-Video Compressor/ does not exist");
            folder.mkdir();
            Log.i("folder", "created");
        }

        String outputFileName = folder.getAbsolutePath() + "/" + video_url.substring(this.video_url.lastIndexOf("/") + 1, this.video_url.lastIndexOf(".")) + "_" + targetFileSize + "_" + quality + ".mp4";
        String logPath = "'" + folder.getAbsolutePath() + "/'";

        // On supprime le fichier s'il existe déjà
        File file = new File(outputFileName);
        if(file.exists()) {
            file.delete();
        }


        // FFMPEG
        List<String> pass1Commands = new LinkedList<>();
        pass1Commands.add("-y");
        pass1Commands.add("-i");
        pass1Commands.add('"' + video_url + '"');
        pass1Commands.add("-codec:v");
        pass1Commands.add("libx264");
        pass1Commands.add("-passlogfile");
        pass1Commands.add(logPath);
        pass1Commands.add("-preset");
        pass1Commands.add(activity.encodeQuality.getSelectedItem().toString());
        pass1Commands.add("-b:v");
        pass1Commands.add(videoBitrate.toString() + "k");
        pass1Commands.add("-maxrate");
        pass1Commands.add(videoBitrate.toString() + "k");
        pass1Commands.add("-bufsize");
        pass1Commands.add("1000k");
        pass1Commands.add("-pass");
        pass1Commands.add("1");
        pass1Commands.add("-an");

        pass1Commands.add("-f");
        pass1Commands.add("null");
        pass1Commands.add("-");

        String ffmpegCommand = "";
        for(String command : pass1Commands) {
            ffmpegCommand += command + " ";
        }
        Log.i("FFMPEG", ffmpegCommand);

        // On lance l'encodage
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(activity, "Encodage en cours...", Toast.LENGTH_SHORT).show();
                activity.encodeVideoBtn.setEnabled(false);
            }
        });

        FFmpegSession firstPass = FFmpegKit.execute(ffmpegCommand);
        if (ReturnCode.isSuccess(firstPass.getReturnCode())) {

            // SUCCESS
            activity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Toast.makeText(activity, "1ère passe terminée", Toast.LENGTH_SHORT).show();
                }
            });
        } else if (ReturnCode.isCancel(firstPass.getReturnCode())) {

            // CANCEL

        } else {
            // FAILURE
            Log.i("Main", String.format("Command failed with state %s and rc %s.%s", firstPass.getState(), firstPass.getReturnCode(), firstPass.getFailStackTrace()));
            activity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Toast.makeText(activity, "Erreur lors de l'encodage de la 1ère passe", Toast.LENGTH_SHORT).show();
                }
            });
        }

        List<String> pass2Commands = new LinkedList<>();
        pass2Commands.add("-i");
        pass2Commands.add('"' + video_url + '"');
        pass2Commands.add("-codec:v");
        pass2Commands.add("libx264");
        pass2Commands.add("-passlogfile");
        pass2Commands.add(logPath);
        pass2Commands.add("-preset");
        pass2Commands.add(activity.encodeQuality.getSelectedItem().toString());
        pass2Commands.add("-b:v");
        pass2Commands.add(videoBitrate.toString() + "k");
        pass2Commands.add("-maxrate");
        pass2Commands.add(videoBitrate.toString() + "k");
        pass2Commands.add("-bufsize");
        pass2Commands.add("1000k");
        pass2Commands.add("-pass");
        pass2Commands.add("2");

        // On regarde si on garde la piste audio
        if(activity.deleteSoundTrack.isChecked()) {
            pass2Commands.add("-an");
        }else{
            pass2Commands.add("-c:a");
            pass2Commands.add("aac");
            pass2Commands.add("-b:a");
            pass2Commands.add(audioBitrate.toString() + "k");
        }

        pass2Commands.add("'" + outputFileName + "'");

        ffmpegCommand = "";
        for(String command : pass2Commands) {
            ffmpegCommand += command + " ";
        }
        Log.i("FFMPEG", ffmpegCommand);

        FFmpegSession secondPass = FFmpegKit.execute(ffmpegCommand);
        if (ReturnCode.isSuccess(firstPass.getReturnCode())) {

            // SUCCESS
            activity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Toast.makeText(activity, "Encodage terminé: " + outputFileName, Toast.LENGTH_LONG).show();
                }
            });
        } else if (ReturnCode.isCancel(secondPass.getReturnCode())) {

            // CANCEL

        } else {
            // FAILURE
            Log.i("Main", String.format("Command failed with state %s and rc %s.%s", secondPass.getState(), secondPass.getReturnCode(), secondPass.getFailStackTrace()));
            activity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    Toast.makeText(activity, "Erreur lors de l'encodage de la 2nd passe", Toast.LENGTH_SHORT).show();
                }
            });
        }
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                activity.encodeVideoBtn.setEnabled(true);
            }
        });
        return null;
    }
}
